@@ -2,13 +2,14 @@
 Extensions to Numpy, including finding array elements and smoothing data.
 
 Highlights:
-    - ``sc.findinds()``: find indices of an array matching a condition
-    - ``sc.findnearest()``: find nearest matching value
-    - ``sc.rolling()``: calculate rolling average
-    - ``sc.smooth()``: simple smoothing of 1D or 2D arrays
+    - :func:`findinds`: find indices of an array matching a condition
+    - :func:`findnearest`: find nearest matching value
+    - :func:`rolling`: calculate rolling average
+    - :func:`smooth`: simple smoothing of 1D or 2D arrays
 '''
 
 import numpy as np
+import pandas as pd
 import warnings
 from . import sc_utils as scu
 
@@ -17,8 +18,8 @@ from . import sc_utils as scu
 #%% Find and approximation functions
 ##############################################################################
 
-__all__ = ['approx', 'safedivide', 'findinds', 'findfirst', 'findlast', 'findnearest',
-           'dataindex', 'getvalidinds', 'sanitize', 'getvaliddata', 'isprime']
+__all__ = ['approx', 'safedivide', 'findinds', 'findfirst', 'findlast', 'findnearest', 'count',
+           'dataindex', 'getvalidinds', 'sanitize', 'rmnans','fillnans', 'getvaliddata', 'isprime', 'numdigits']
 
 
 def approx(val1=None, val2=None, eps=None, **kwargs):
@@ -81,36 +82,42 @@ def safedivide(numerator=None, denominator=None, default=None, eps=None, warn=Fa
     return output
 
 
-def findinds(arr=None, val=None, eps=1e-6, first=False, last=False, die=True, **kwargs):
+def findinds(arr=None, val=None, *args, eps=1e-6, first=False, last=False, ind=None, die=True, **kwargs):
     '''
-    Little function to find matches even if two things aren't eactly equal (eg.
-    due to floats vs. ints). If one argument, find nonzero values. With two arguments,
-    check for equality using eps. Returns a tuple of arrays if val1 is multidimensional,
-    else returns an array. Similar to calling np.nonzero(np.isclose(arr, val))[0].
+    Find matches even if two things aren't eactly equal (e.g. floats vs. ints).
+
+    If one argument, find nonzero values. With two arguments, check for equality
+    using eps. Returns a tuple of arrays if val1 is multidimensional, else returns
+    an array. Similar to calling ``np.nonzero(np.isclose(arr, val))[0]``.
 
     Args:
-        arr (array): the array to find values in
-        val (float): if provided, the value to match
-        eps (float): the precision for matching (default 1e-6, equivalent to np.isclose's atol)
-        first (bool): whether to return the first matching value
-        last (bool): whether to return the last matching value
-        die (bool): whether to raise an exception if first or last is true and no matches were found
-        kwargs (dict): passed to np.isclose()
+        arr    (array): the array to find values in
+        val    (float): if provided, the value to match
+        args   (list):  if provided, additional boolean arrays
+        eps    (float): the precision for matching (default 1e-6, equivalent to ``np.isclose()``'s atol)
+        first  (bool):  whether to return the first matching value (equivalent to ind=0)
+        last   (bool):  whether to return the last matching value (equivalent to ind=-1)
+        ind    (int):   index of match to retrieve
+        die    (bool):  whether to raise an exception if first or last is true and no matches were found
+        kwargs (dict):  passed to ``np.isclose()``
 
     **Examples**::
 
-        sc.findinds(rand(10)<0.5) # returns e.g. array([2, 4, 5, 9])
-        sc.findinds([2,3,6,3], 3) # returs array([1,3])
-        sc.findinds([2,3,6,3], 3, first=True) # returns 1
+        data = np.random.rand(10)
+        sc.findinds(data<0.5) # Standard usage; returns e.g. array([2, 4, 5, 9])
+        sc.findinds(data>0.1, data<0.5) # Multiple arguments
 
-    New in version 1.2.3: "die" argument
+        sc.findinds([2,3,6,3], 3) # Returs array([1,3])
+        sc.findinds([2,3,6,3], 3, first=True) # Returns 1
+
+    | New in version 1.2.3: "die" argument
+    | New in version 2.0.0: fix string matching; allow multiple arguments
     '''
 
     # Handle first or last
     if first and last: raise ValueError('Can use first or last but not both')
     elif first: ind = 0
     elif last:  ind = -1
-    else:       ind = None
 
     # Handle kwargs
     atol = kwargs.pop('atol', eps) # Ensure atol isn't specified twice
@@ -118,22 +125,35 @@ def findinds(arr=None, val=None, eps=1e-6, first=False, last=False, die=True, **
         arr = kwargs.pop('val1', arr)
         val = kwargs.pop('val2', val)
         warnmsg = 'sc.findinds() arguments "val1" and "val2" have been deprecated as of v1.0.0; use "arr" and "val" instead'
-        warnings.warn(warnmsg, category=DeprecationWarning, stacklevel=2)
+        warnings.warn(warnmsg, category=FutureWarning, stacklevel=2)
 
     # Calculate matches
     arr = scu.promotetoarray(arr)
+    arglist = list(args)
     if val is None: # Check for equality
-        output = np.nonzero(arr) # If not, just check the truth condition
+        boolarr = arr # If not, just check the truth condition
     else:
-        if scu.isstring(val):
-            output = np.nonzero(arr==val)
-        try: # Standard usage, use nonzero
-            output = np.nonzero(np.isclose(a=arr, b=val, atol=atol, **kwargs)) # If absolute difference between the two values is less than a certain amount
-        except Exception as E: # pragma: no cover # As a fallback, try simpler comparison
-            output = np.nonzero(abs(arr-val) < atol)
-            if kwargs: # Raise a warning if and only if special settings were passed
-                warnmsg = f'{str(E)}\nsc.findinds(): np.isclose() encountered an exception (above), falling back to direct comparison'
-                warnings.warn(warnmsg, category=RuntimeWarning, stacklevel=2)
+        if scu.isstring(val): # A string only matches itself
+            boolarr = (arr == val)
+        else:
+            if scu.isnumber(val): # Standard usage, use nonzero
+                boolarr = np.isclose(a=arr, b=val, atol=atol, **kwargs) # If absolute difference between the two values is less than a certain amount
+            elif scu.checktype(val, 'arraylike'): # It's not actually a value, it's another array
+                boolarr = arr
+                arglist.append(scu.promotetoarray(val))
+            else:
+                errormsg = f'Cannot understand input {type(val)}: must be number or array-like'
+                raise TypeError(errormsg)
+
+    # Handle any additional inputs
+    for arg in arglist:
+        if arg.shape != boolarr.shape:
+            errormsg = f'Could not handle inputs with shapes {boolarr.shape} vs {arg.shape}'
+            raise ValueError(errormsg)
+        boolarr *= arg
+
+    # Actually find indices
+    output = np.nonzero(boolarr)
 
     # Process output
     try:
@@ -189,6 +209,31 @@ def findnearest(series=None, value=None):
     return output
 
 
+def count(arr=None, val=None, eps=1e-6, **kwargs):
+    '''
+    Count the number of matching elements.
+
+    Similar to ``np.count_nonzero()``, but allows for slight mismatches (e.g.,
+    floats vs. ints). Equivalent to ``len(sc.findinds())``.
+
+    Args:
+        arr (array): the array to find values in
+        val (float): if provided, the value to match
+        eps (float): the precision for matching (default 1e-6, equivalent to np.isclose's atol)
+        kwargs (dict): passed to ``np.isclose()``
+
+    **Examples**::
+
+        sc.count(rand(10)<0.5) # returns e.g. 4
+        sc.count([2,3,6,3], 3) # returs 2
+
+    New in version 2.0.0.
+    '''
+    output = len(findinds(arr=arr, val=val, eps=eps, **kwargs))
+    return output
+
+
+
 def dataindex(dataarray, index): # pragma: no cover
     '''
     Take an array of data and return either the first or last (or some other) non-NaN entry.
@@ -253,41 +298,77 @@ def getvaliddata(data=None, filterdata=None, defaultind=0): # pragma: no cover
     return validdata
 
 
-def sanitize(data=None, returninds=False, replacenans=None, die=True, defaultval=None, label=None, verbose=True):
+def sanitize(data=None, returninds=False, replacenans=None, defaultval=None, die=True, verbose=False, label=None):
         '''
-        Sanitize input to remove NaNs. Warning, does not work on multidimensional data!!
+        Sanitize input to remove NaNs. (NB: ``sc.sanitize()`` and ``sc.rmnans()`` are aliases.)
+
+        Returns an array with the sanitized data. If ``replacenans=True``, the sanitized
+        array is of the same length/size as data. If ``replacenans=False``, the sanitized
+        array may be shorter than data.
+
+        Args:
+            data        (arr/list)   : array or list with numbers to be sanitized
+            returninds  (bool)       : whether to return indices of non-nan/valid elements, indices are with respect the shape of data
+            replacenans (float/str)  : whether to replace the NaNs with the specified value, or if ``True`` or a string, using interpolation
+            defaultval  (float)      : value to return if the sanitized array is empty
+            die         (bool)       : whether to raise an exception if the sanitization failed (otherwise return an empty array)
+            verbose     (bool)       : whether to print out a warning if no valid values are found
+            label       (str)        : human readable label for data (for use with verbose mode only)
 
         **Examples**::
 
-            sanitized,inds = sanitize(array([3,4,nan,8,2,nan,nan,nan,8]), returninds=True)
-            sanitized = sanitize(array([3,4,nan,8,2,nan,nan,nan,8]), replacenans=True)
-            sanitized = sanitize(array([3,4,nan,8,2,nan,nan,nan,8]), replacenans=0)
+            data = [3, 4, np.nan, 8, 2, np.nan, np.nan, 8]
+            sanitized1, inds = sc.sanitize(data, returninds=True) # Remove NaNs
+            sanitized2 = sc.sanitize(data, replacenans=True) # Replace NaNs using nearest neighbor interpolation
+            sanitized3 = sc.sanitize(data, replacenans='nearest') # Eequivalent to replacenans=True
+            sanitized4 = sc.sanitize(data, replacenans='linear') # Replace NaNs using linear interpolation
+            sanitized5 = sc.sanitize(data, replacenans=0) # Replace NaNs with 0
+
+        New in version 2.0.0: handle multidimensional arrays
         '''
         try:
-            data = np.array(data,dtype=float) # Make sure it's an array of float type
-            inds = np.nonzero(~np.isnan(data))[0] # WARNING, nonzero returns tuple :(
-            sanitized = data[inds] # Trim data
+            data = np.array(data, dtype=float) # Make sure it's an array of float type
+            is_multidim = data.ndim > 1
+            if is_multidim:
+                if not replacenans:
+                    errormsg = 'For multidimensional data, NaNs cannot be removed. Set replacenans=<value>, or flatten data before use.'
+                    raise ValueError(errormsg)
+            inds = np.nonzero(~np.isnan(data))
+            if not is_multidim:
+                inds = inds[0] # Since np.nonzero() returns a tuple
+                sanitized = data[inds] # Trim data
+
             if replacenans is not None:
-                newx = range(len(data)) # Create a new x array the size of the original array
-                if replacenans==True: replacenans = 'nearest'
-                if replacenans in ['nearest','linear']:
-                    sanitized = smoothinterp(newx, inds, sanitized, method=replacenans, smoothness=0) # Replace nans with interpolated values
+                if replacenans is True:
+                    replacenans = 'nearest'
+                if scu.isstring(replacenans):
+                    if replacenans in ['nearest','linear']:
+                        if is_multidim:
+                            errormsg = 'Cannot perform interpolation on multidimensional data; use replacenans=<value> instead'
+                            raise NotImplementedError(errormsg)
+                        newx = range(len(data)) # Create a new x array the size of the original array
+                        sanitized = smoothinterp(newx, inds, sanitized, method=replacenans, smoothness=0) # Replace nans with interpolated values
+                    else:
+                        errormsg = f'Interpolation method "{replacenans}" not found: must be "nearest" or "linear"'
+                        raise ValueError(errormsg)
                 else:
-                    naninds = inds = np.nonzero(np.isnan(data))[0]
-                    sanitized = scu.dcp(data)
+                    naninds = np.nonzero(np.isnan(data))
+                    sanitized = data.copy() # To avoid overwriting original array
                     sanitized[naninds] = replacenans
+
             if len(sanitized)==0:
                 if defaultval is not None:
                     sanitized = defaultval
                 else:
-                    sanitized = 0.0
+                    sanitized = data
+                    inds = []
+
                     if verbose: # pragma: no cover
-                        if label is None: label = 'this parameter'
-                        print(f'sc.sanitize(): no data entered for {label}, assuming 0')
+                        if label is None: label = 'these input data'
+                        print(f'sc.sanitize(): no valid values found for {label}. Returning 0.')
         except Exception as E: # pragma: no cover
             if die:
-                errormsg = f'Sanitization failed on array: "{repr(E)}":\n{data}'
-                raise RuntimeError(errormsg)
+                raise E
             else:
                 sanitized = data # Give up and just return an empty array
                 inds = []
@@ -295,6 +376,19 @@ def sanitize(data=None, returninds=False, replacenans=None, die=True, defaultval
         else:          return sanitized
 
 
+def fillnans(data=None, replacenans=True, **kwargs):
+    """
+    Alias for ``sc.sanitize(..., replacenans=True) with nearest interpolation (or a specified value).
+
+    New in version 2.0.0.
+    """
+    return sanitize(data=data, replacenans=replacenans, **kwargs)
+fillnans.__doc__ += '\n\n' + sanitize.__doc__
+
+# Define as an alias
+rmnans = sanitize
+
+
 def isprime(n, verbose=False):
     '''
     Determine if a number is prime.
@@ -337,6 +431,65 @@ def isprime(n, verbose=False):
     return True
 
 
+def numdigits(n, *args, count_minus=False, count_decimal=False):
+    """
+    Count the number of digits in a number (or list of numbers).
+
+    Useful for e.g. knowing how long a string needs to be to fit a given number.
+
+    If a number is less than 1, return the number of digits until the decimal
+    place.
+
+    Reference: https://stackoverflow.com/questions/22656345/how-to-count-the-number-of-digits-in-python
+
+    Args:
+        n (int/float/list/array): number or list of numbers
+        args (list): additional numbers
+        count_minus (bool): whether to count the minus sign as a digit
+        count_decimal (bool): whether to count the decimal point as a digit
+
+    **Examples**::
+
+        sc.numdigits(12345) # Returns 5
+        sc.numdigits(12345.5) # Returns 5
+        sc.numdigits(0) # Returns 1
+        sc.numdigits(-12345) # Returns 5
+        sc.numdigits(-12345, count_minus=True) # Returns 6
+        sc.numdigits(12, 123, 12345) # Returns [2, 3, 5]
+        sc.numdigits(0.01) # Returns -2
+        sc.numdigits(0.01, count_decimal=True) # Returns -4
+
+    New in version 2.0.0.
+    """
+    is_scalar = True if scu.isnumber(n) and len(args) == 0 else False
+
+    vals = cat(n, *args)
+
+    output = []
+    for n in vals:
+        abs_n = abs(n)
+        is_decimal = 0 < abs_n < 1
+        n_digits = 1
+        if n < 0 and count_minus:
+            n_digits += 1
+        if is_decimal:
+            if count_decimal:
+                n_digits += 1
+            else:
+                n_digits -= 1
+
+        if abs_n > 0:
+            if is_decimal:
+                n_digits = -n_digits
+            n_digits += int(np.floor(np.log10(abs_n)))
+        output.append(n_digits)
+    output = np.array(output)
+    if is_scalar:
+        output = output[0]
+
+    return output
+
+
 
 ##############################################################################
 #%% Other functions
@@ -352,6 +505,7 @@ def perturb(n=1, span=0.5, randseed=None, normal=False):
     Args:
         n (int): number of points
         span (float): width of distribution on either side of 1
+        randseed (int): seed passed to the reseed numpy's legacy MT19937 BitGenerator
         normal (bool):  whether to use a normal distribution instead of uniform
 
     **Example**::
@@ -493,16 +647,14 @@ def randround(x):
     return output
 
 
-def cat(*args, axis=None, copy=False, **kwargs):
+def cat(*args, copy=False, **kwargs):
     '''
     Like np.concatenate(), but takes anything and returns an array. Useful for
     e.g. appending a single number onto the beginning or end of an array.
 
     Args:
         args   (any):  items to concatenate into an array
-        axis   (int):  axis along which to concatenate
-        copy   (bool): whether or not to deepcopy the result
-        kwargs (dict): passed to ``np.array()``
+        kwargs (dict): passed to ``np.concatenate()``
 
     **Examples**::
 
@@ -512,16 +664,12 @@ def cat(*args, axis=None, copy=False, **kwargs):
 
     | New in version 1.0.0.
     | New in version 1.1.0: "copy" and keyword arguments.
+    | New in version 2.0.2: removed "copy" argument; changed default axis of 0; arguments passed to ``np.concatenate()``
     '''
     if not len(args):
         return np.array([])
-    output = scu.promotetoarray(args[0])
-    for arg in args[1:]:
-        arg = scu.promotetoarray(arg)
-        output = np.concatenate((output, arg), axis=axis)
-    output = np.array(output, **kwargs)
-    if copy:
-        output = scu.dcp(output)
+    arrs = [scu.promotetoarray(arg) for arg in args] # Key step: convert everything to an array
+    output = np.concatenate(arrs, **kwargs)
     return output
 
 
@@ -536,7 +684,7 @@ __all__ += ['rolling', 'convolve', 'smooth', 'smoothinterp', 'gauss1d', 'gauss2d
 
 def rolling(data, window=7, operation='mean', **kwargs):
     '''
-    Alias to Pandas' rolling() (window) method to smooth a series.
+    Alias to pandas' rolling() (window) method to smooth a series.
 
     Args:
         data (list/arr): the 1D or 2D data to be smoothed
@@ -549,8 +697,6 @@ def rolling(data, window=7, operation='mean', **kwargs):
         data = [5,5,5,0,0,0,0,7,7,7,7,0,0,3,3,3]
         rolled = sc.rolling(data)
     '''
-    import pandas as pd # Optional import
-
     # Handle the data
     data = np.array(data)
     data = pd.Series(data) if data.ndim == 1 else pd.DataFrame(data)
@@ -647,10 +793,10 @@ def smooth(data, repeats=None, kernel=None, legacy=False):
     if kernel is None:
         kernel = [0.25,0.5,0.25]
     kernel = np.array(kernel)
-    output = scu.dcp(np.array(data))
+    output = np.array(data).copy()
 
     # Only convolve the kernel with itself -- equivalent to doing the full convolution multiple times
-    v = scu.dcp(kernel)
+    v = kernel.copy()
     for r in range(repeats-1):
         v = np.convolve(v, kernel, mode='full')
 
@@ -1021,4 +1167,4 @@ def gauss2d(x=None, y=None, z=None, xi=None, yi=None, scale=1.0, xscale=1.0, ysc
     # Convert back to original size and dtype
     zi = np.array(zi.reshape(orig_shape), dtype=orig_dtype)
 
-    return zi
\ No newline at end of file
+    return zi
