@@ -2,12 +2,12 @@
 The 'odict' class, combining features from an OrderedDict and a list/array.
 
 Highlights:
-    - ``sc.odict()``: flexible container representing the best-of-all-worlds across lists, dicts, and arrays
-    - ``sc.objdict()``: like an odict, but allows get/set via e.g. ``foo.bar`` instead of ``foo['bar']``
+    - :class:`odict`: flexible container representing the best-of-all-worlds across lists, dicts, and arrays
+    - :class:`objdict`: like an odict, but allows get/set via e.g. ``foo.bar`` instead of ``foo['bar']``
 '''
 
 ##############################################################################
-### ODICT CLASS
+#%% odict class
 ##############################################################################
 
 import re
@@ -18,7 +18,7 @@ from . import sc_printing as scp
 from . import sc_nested as scn
 
 # Restrict imports to user-facing modules
-__all__ = ['ddict', 'odict', 'objdict', 'asobj']
+__all__ = ['ddict', 'odict', 'objdict', 'dictobj', 'asobj']
 
 
 class odict(OD):
@@ -71,6 +71,7 @@ class odict(OD):
 
     | New in version 1.1.0: "defaultdict" argument
     | New in version 1.3.1: allow integer keys via ``makefrom()``; removed ``to_OD``; performance improvements
+    | New in version 2.0.1: allow deletion by index
     '''
 
     def __init__(self, *args, defaultdict=None, **kwargs):
@@ -83,7 +84,7 @@ class odict(OD):
                 raise TypeError(errormsg)
             self._setattr('_defaultdict', defaultdict) # Use OD.__setattr__() since setattr() is overridden by sc.objdict()
         self._cache_keys()
-        return None
+        return
 
 
     def _cache_keys(self):
@@ -160,7 +161,7 @@ class odict(OD):
                 return output
 
             else: # pragma: no cover # Handle everything else (rare)
-                return OD.__getitem__(self,key)
+                return OD.__getitem__(self, key)
 
 
     def __setitem__(self, key, value):
@@ -358,10 +359,17 @@ class odict(OD):
         else:         return self.__add__(dict2)
 
 
-    def __delitem__(self, *args, **kwargs):
-        ''' Default delitem, except set stale to true '''
+    def __delitem__(self, key):
+        ''' Default delitem, except set stale to true and allow numeric values; slices etc are not supported '''
         self._setattr('_stale', True) # Flag to refresh the cached keys
-        return OD.__delitem__(self, *args, **kwargs)
+        try:
+            return OD.__delitem__(self, key)
+        except Exception as E:
+            if isinstance(key, scu._numtype): # If it's a number, use that
+                thiskey = self._ikey(key)
+                return OD.__delitem__(self, thiskey) # Note that defaultdict behavior isn't supported for non-string lookup
+            else:
+                raise E
 
 
     def disp(self, maxlen=None, showmultilines=True, divider=False, dividerthresh=10, numindents=0, sigfigs=5, numformat=None, maxitems=20, **kwargs):
@@ -376,7 +384,7 @@ class odict(OD):
         '''
         kwargs = scu.mergedicts(dict(maxlen=maxlen, showmultilines=showmultilines, divider=divider, dividerthresh=dividerthresh, numindents=numindents, recursionlevel=0, sigfigs=sigfigs, numformat=None, maxitems=maxitems), kwargs)
         print(self.__repr__(**kwargs))
-        return None
+        return
 
 
     def _ikey(self, key):
@@ -473,7 +481,7 @@ class odict(OD):
         output += end
         if doprint:
             print(output)
-            return None
+            return
         else:
             return output
 
@@ -522,14 +530,14 @@ class odict(OD):
     def remove(self, key, *args, **kwargs):
         ''' Remove an item by key and do not return it '''
         self.pop(key, *args, **kwargs)
-        return None
+        return
 
 
     def clear(self):
         ''' Reset to an empty odict '''
         for key in self.keys():
             self.remove(key)
-        return None
+        return
 
 
     def index(self, value):
@@ -631,7 +639,7 @@ class odict(OD):
         else:
             keyname = key
         self.__setitem__(keyname, value)
-        return None
+        return
 
 
     def insert(self, pos=None, key=None, value=None):
@@ -682,18 +690,18 @@ class odict(OD):
             for keytopop in keystopop: # Insert popped items back in
                 self.__setitem__(keytopop, tmpdict.pop(keytopop))
 
-        return None
+        return
 
 
     def copy(self, oldkey, newkey):
         ''' Make a copy of an item '''
         newval = scu.dcp(self.__getitem__(oldkey))
         self.__setitem__(newkey, newval)
-        return None
+        return
 
 
     def rename(self, oldkey, newkey):
-        ''' Change a key name -- WARNING, very inefficient! '''
+        ''' Change a key name (note: not optimized for speed) '''
         nkeys = len(self)
         if isinstance(oldkey, scu._numtype):
             index = oldkey
@@ -707,7 +715,7 @@ class odict(OD):
                 key = self.keys()[index]
                 value = self.pop(key)
                 self.__setitem__(key, value)
-        return None
+        return
 
 
     def sort(self, sortby=None, reverse=False, copy=False, verbose=True):
@@ -749,7 +757,7 @@ class odict(OD):
         else:
             for key in allkeys: tmpdict.__setitem__(key, self.pop(key))
             for key in allkeys: self.__setitem__(key, tmpdict.pop(key))
-            return None
+            return
 
 
     def sorted(self, sortby=None, reverse=False):
@@ -954,7 +962,7 @@ class odict(OD):
             raise ValueError(errormsg)
         for k,key in self.enumkeys():
             self.__getitem__(key)[ind] = val[k]
-        return None
+        return
 
 
     def enumkeys(self, transpose=False):
@@ -1034,6 +1042,18 @@ class odict(OD):
         if transpose: iterator = tuple(scu.transposelist(iterator))
         return iterator
 
+    def dict_keys(self):
+        """ Return an iterator (not a list) over keys (as in Python 2). """
+        return OD.keys(self)
+
+    def dict_values(self):
+        """ Return an iterator (not a list) over values (as in Python 2). """
+        return OD.values(self)
+
+    def dict_items(self):
+        """ Return an iterator (not a list) over items (as in Python 2). """
+        return OD.items(self)
+
     def iteritems(self, transpose=False):
         """ Alias to items() """
         return self.items(transpose=transpose)
@@ -1060,6 +1080,8 @@ class objdict(odict):
     An ``odict`` that acts like an object -- allow keys to be set/retrieved by object
     notation.
 
+    For a lighter-weight example, see ``sc.dictobj()``.
+
     Example
     -------
     >>> import sciris as sc
@@ -1173,12 +1195,56 @@ class objdict(odict):
 
     def delattribute(self, name):
         ''' Delete attribute if truly desired '''
-        try:
-            del self[name]
-        except:
-            odict.__delattr__(self, name)
+        return odict.__delattr__(self, name)
+
+
+class dictobj(dict):
+    '''
+    Lightweight class to create an object that can also act like a dictionary.
+
+    **Example**::
+
+        obj = sc.dictobj()
+        obj.a = 5
+        obj['b'] = 10
+        print(obj.items())
+
+    For a more powerful alternative, see ``sc.objdict()``.
+
+    | New in version 1.3.0.
+    | New in version 1.3.1: inherit from dict
+    | New in version 2.0.0: allow positional arguments
+    '''
+
+    def __init__(self, *args, **kwargs):
+        kwargs = scu.mergedicts(*args, kwargs)
+        for k,v in kwargs.items():
+            self.__dict__[k] = v
         return
 
+    def __repr__(self):
+        output = 'dictobj(' + self.__dict__.__repr__() + ')'
+        return output
+
+    def fromkeys(self, *args, **kwargs):
+        return dictobj(self.__dict__.fromkeys(*args, **kwargs))
+
+    # Copy default dictionary methods
+    def __getitem__( self, *args, **kwargs): return self.__dict__.__getitem__( *args, **kwargs)
+    def __setitem__( self, *args, **kwargs): return self.__dict__.__setitem__( *args, **kwargs)
+    def __contains__(self, *args, **kwargs): return self.__dict__.__contains__(*args, **kwargs)
+    def __len__(     self, *args, **kwargs): return self.__dict__.__len__(     *args, **kwargs)
+    def clear(       self, *args, **kwargs): return self.__dict__.clear(       *args, **kwargs)
+    def copy(        self, *args, **kwargs): return self.__dict__.copy(        *args, **kwargs)
+    def get(         self, *args, **kwargs): return self.__dict__.get(         *args, **kwargs)
+    def items(       self, *args, **kwargs): return self.__dict__.items(       *args, **kwargs)
+    def keys(        self, *args, **kwargs): return self.__dict__.keys(        *args, **kwargs)
+    def pop(         self, *args, **kwargs): return self.__dict__.pop(         *args, **kwargs)
+    def popitem(     self, *args, **kwargs): return self.__dict__.popitem(     *args, **kwargs)
+    def setdefault(  self, *args, **kwargs): return self.__dict__.setdefault(  *args, **kwargs)
+    def update(      self, *args, **kwargs): return self.__dict__.update(      *args, **kwargs)
+    def values(      self, *args, **kwargs): return self.__dict__.values(      *args, **kwargs)
+
 
 def asobj(obj, strict=True):
     '''
