@@ -2,21 +2,26 @@
 Printing/notification functions.
 
 Highlights:
-    - ``sc.pr()``: print full representation of an object, including methods and each attribute
-    - ``sc.heading()``: print text as a 'large' heading
-    - ``sc.colorize()``: print text in a certain color
-    - ``sc.sigfigs()``: truncate a number to a certain number of significant figures
-    - ``sc.progressbar()``: show a (text-based) progress bar
+    - :func:`heading`: print text as a 'large' heading
+    - :func:`colorize`: print text in a certain color
+    - :func:`pr`: print full representation of an object, including methods and each attribute
+    - :func:`sigfigs`: truncate a number to a certain number of significant figures
+    - :func:`progressbar`: show a (text-based) progress bar
+    - :func:`capture`: capture text output (e.g., stdout) as a variable
 '''
 
+import io
 import os
+import sys
 import time
-import colors
 import pprint
 import numpy as np
 import collections as co
 from textwrap import fill
+from collections import UserString
+from contextlib import redirect_stdout
 from . import sc_utils as scu
+from . import ansicolors as ac
 
 
 # Add Windows support for colors (do this at the module level so that colorama.init() only gets called once)
@@ -31,45 +36,11 @@ else:
     ansi_support = True
 
 
-__all__ = ['printv', 'blank', 'createcollist', 'objectid', 'objatt', 'objmeth', 'objprop', 'objrepr',
-            'prepr', 'pr', 'indent', 'sigfig', 'printarr', 'printdata', 'printvars',
-            'slacknotification', 'printtologfile', 'colorize', 'heading', 'percentcomplete', 'progressbar']
-
-def printv(string, thisverbose=1, verbose=2, newline=True, indent=True):
-    '''
-    Optionally print a message and automatically indent. The idea is that
-    a global or shared "verbose" variable is defined, which is passed to
-    subfunctions, determining how much detail to print out.
-
-    The general idea is that verbose is an integer from 0-4 as follows:
-
-    * 0 = no printout whatsoever
-    * 1 = only essential warnings, e.g. suppressed exceptions
-    * 2 = standard printout
-    * 3 = extra debugging detail (e.g., printout on each iteration)
-    * 4 = everything possible (e.g., printout on each timestep)
-
-    Thus a very important statement might be e.g.
-
-    >>> sc.printv('WARNING, everything is wrong', 1, verbose)
-
-    whereas a much less important message might be
-
-    >>> sc.printv(f'This is timestep {i}', 4, verbose)
-
-    Version: 2016jan30
-    '''
-    if thisverbose>4 or verbose>4: print(f'Warning, verbosity should be from 0-4 (this message: {thisverbose}; current: {verbose})')
-    if verbose>=thisverbose: # Only print if sufficiently verbose
-        indents = '  '*thisverbose*bool(indent) # Create automatic indenting
-        if newline: print(indents+scu.flexstr(string)) # Actually print
-        else: print(indents+scu.flexstr(string)), # Actually print
-    return None
 
 
-def blank(n=3):
-    ''' Tiny function to print n blank lines, 3 by default '''
-    print('\n'*n)
+#%% Object display functions
+
+__all__ = ['createcollist', 'objectid', 'objatt', 'objmeth', 'objprop', 'objrepr', 'prepr', 'pr']
 
 
 def createcollist(items, title=None, strlen=18, ncol=3):
@@ -167,7 +138,7 @@ def prepr(obj, maxlen=None, maxitems=None, skip=None, dividerchar='—', divider
         maxitems (int): maximum number of items to show in the object
         skip (list): any properties to skip
         dividerchar (str): divider for methods, attributes, etc.
-        divierlen (int): number of divider characters
+        dividerlen (int): number of divider characters
         use_repr (bool): whether to use repr() or str() to parse the object
         maxtime (float): maximum amount of time to spend on trying to print the object
         die (bool): whether to raise an exception if an error is encountered
@@ -282,7 +253,17 @@ def pr(obj, *args, **kwargs):
         sc.pr(df) # See all the methods too
     '''
     print(prepr(obj, *args, **kwargs))
-    return None
+    return
+
+
+#%% Spacing functions
+
+__all__ += ['blank', 'indent']
+
+
+def blank(n=3):
+    ''' Tiny function to print n blank lines, 3 by default '''
+    print('\n'*n)
 
 
 def indent(prefix=None, text=None, suffix='\n', n=0, pretty=False, width=70, **kwargs):
@@ -339,6 +320,11 @@ def indent(prefix=None, text=None, suffix='\n', n=0, pretty=False, width=70, **k
 
 
 
+#%% Data representation functions
+
+__all__ += ['sigfig', 'printarr', 'printdata', 'printvars']
+
+
 def sigfig(x, sigfigs=5, SI=False, sep=False, keepints=False):
     '''
     Return a string representation of variable x with sigfigs number of significant figures
@@ -436,7 +422,7 @@ def printarr(arr, arrformat='%0.2f  '):
                 printarr(arr[i][j], arrformat)
     else: # pragma: no cover
         print(arr) # Give up
-    return None
+    return
 
 
 
@@ -492,7 +478,7 @@ def printdata(data, name='Variable', depth=1, maxlen=40, indent='', level=0, sho
                     thisindent = ' '*(maxkeylen-len(key))
                     printdata(getattr(data,key), name=key, depth=depth-1, indent=indent+thisindent, level=level)
         print('\n')
-    return None
+    return
 
 
 def printvars(localvars=None, varlist=None, label=None, divider=True, spaces=1, color=None):
@@ -534,107 +520,14 @@ def printvars(localvars=None, varlist=None, label=None, divider=True, spaces=1,
         controlstr += '\n' * spaces # The number of spaces to add between variables
         print(controlstr), # Print it out
     if divider: print(dividerstr) # If necessary, print the divider again
-    return None
+    return
 
 
-def slacknotification(message=None, webhook=None, to=None, fromuser=None, verbose=2, die=False):  # pragma: no cover
-    '''
-    Send a Slack notification when something is finished.
 
-    The webhook is either a string containing the webhook itself, or a plain text file containing
-    a single line which is the Slack webhook. By default it will look for the file
-    ".slackurl" in the user's home folder. The webhook needs to look something like
-    "https://hooks.slack.com/services/af7d8w7f/sfd7df9sb/lkcpfj6kf93ds3gj". Webhooks are
-    effectively passwords and must be kept secure! Alternatively, you can specify the webhook
-    in the environment variable SLACKURL.
+#%% Color functions
 
-    Args:
-        message (str): The message to be posted.
-        webhook (str): See above
-        to (str): The Slack channel or user to post to. Channels begin with #, while users begin with @ (note: ignored by new-style webhooks)
-        fromuser (str): The pseudo-user the message will appear from (note: ignored by new-style webhooks)
-        verbose (bool): How much detail to display.
-        die (bool): If false, prints warnings. If true, raises exceptions.
-
-    **Example**::
-
-        sc.slacknotification('Long process is finished')
-        sc.slacknotification(webhook='/.slackurl', channel='@username', message='Hi, how are you going?')
-
-    What's the point? Add this to the end of a very long-running script to notify
-    your loved ones that the script has finished.
-
-    Version: 2018sep25
-    '''
-    try:
-        from requests import post # Simple way of posting data to a URL
-        from json import dumps # For sanitizing the message
-    except Exception as E:
-        errormsg = f'Cannot use Slack notification since imports failed: {str(E)}'
-        if die: raise ImportError(errormsg)
-        else:   print(errormsg)
-
-    # Validate input arguments
-    printv('Sending Slack message', 1, verbose)
-    if not webhook:  webhook    = os.path.expanduser('~/.slackurl')
-    if not to:       to       = '#general'
-    if not fromuser: fromuser = 'sciris-bot'
-    if not message:  message  = 'This is an automated notification: your notifier is notifying you.'
-    printv(f'Channel: {to} | User: {fromuser} | Message: {message}', 3, verbose) # Print details of what's being sent
-
-    # Try opening webhook as a file
-    if webhook.find('hooks.slack.com')>=0: # It seems to be a URL, let's proceed
-        slackurl = webhook
-    elif os.path.exists(os.path.expanduser(webhook)): # If not, look for it sa a file
-        with open(os.path.expanduser(webhook)) as f: slackurl = f.read()
-    elif os.getenv('SLACKURL'): # See if it's set in the user's environment variables
-        slackurl = os.getenv('SLACKURL')
-    else:
-        slackurl = webhook # It doesn't seemt to be a URL but let's try anyway
-        errormsg = f'"{webhook}" does not seem to be a valid webhook string or file'
-        if die: raise ValueError(errormsg)
-        else:   print(errormsg)
-
-    # Package and post payload
-    try:
-        payload = '{"text": %s, "channel": %s, "username": %s}' % (dumps(message), dumps(to), dumps(fromuser))
-        printv(f'Full payload: {payload}', 4, verbose)
-        response = post(url=slackurl, data=payload)
-        printv(response, 3, verbose) # Optionally print response
-        printv('Message sent.', 2, verbose) # We're done
-    except Exception as E:
-        errormsg = f'Sending of Slack message failed: {repr(E)}'
-        if die: raise RuntimeError(errormsg)
-        else:   print(errormsg)
-    return None
-
-
-def printtologfile(message=None, filename=None):
-    '''
-    Append a message string to a file specified by a filename name/path.  This
-    is especially useful for capturing information from spawned processes not
-    so handily captured through print statements.
-
-    Warning: If you pass a file in, existing or not, it will try to append
-    text to it!
-    '''
-
-    # Set defaults
-    if message is None:
-        return None # Return immediately if nothing to append
-    if filename is None:
-        import tempfile
-        tempdir = tempfile.gettempdir()
-        filename = os.path.join(tempdir, 'logfile') # Some generic filename that should work on *nix systems
-
-    # Try writing to file
-    try:
-        with open(filename, 'a') as f:
-            f.write('\n'+message+'\n') # Add a newline to the message.
-    except Exception as E: # pragma: no cover # Fail gracefully
-        print(f'Warning, could not write to logfile {filename}: {str(E)}')
-
-    return None
+__all__ += ['colorize', 'heading', 'printred', 'printyellow', 'printgreen',
+            'printcyan', 'printblue', 'printmagenta']
 
 
 def colorize(color=None, string=None, doprint=None, output=False, enable=True, showhelp=False, fg=None, bg=None, style=None):
@@ -648,6 +541,9 @@ def colorize(color=None, string=None, doprint=None, output=False, enable=True, s
         output (bool): whether to return the modified version of the string (default false)
         enable (bool): switch to allow ``sc.colorize()`` to be easily turned off without converting to a ``print()`` statement
         showhelp (bool): show help rather than changing colors
+        fg (str): foreground colour
+        bg (str): background colour
+        style (str): font style (eg, italic, underline, bold)
 
     **Examples**::
 
@@ -669,7 +565,7 @@ def colorize(color=None, string=None, doprint=None, output=False, enable=True, s
             return string
         else:
             print(string)
-            return None
+            return
 
     # Decide which path we'll be taking
     ansistring = ''
@@ -683,7 +579,7 @@ def colorize(color=None, string=None, doprint=None, output=False, enable=True, s
             errormsg = 'You can supply either color or fg, but not both'
             raise ValueError(errormsg)
 
-        if ansi_support: ansistring = colors.color(s=string, fg=fg, bg=bg, style=style) # Actually apply color
+        if ansi_support: ansistring = ac.color(s=string, fg=fg, bg=bg, style=style) # Actually apply color
         else:            ansistring = str(string) # Otherwise, just return the string
 
     # Original use case
@@ -717,7 +613,7 @@ def colorize(color=None, string=None, doprint=None, output=False, enable=True, s
         for color in colorlist:
             if color not in ansicolors.keys(): # pragma: no cover
                 print(f'Color "{color}" is not available, use colorize(showhelp=True) to show options.')
-                return None  # Don't proceed if the color isn't found
+                return  # Don't proceed if the color isn't found
         ansicolor = ''
         for color in colorlist:
             ansicolor += ansicolors[color]
@@ -743,6 +639,32 @@ def colorize(color=None, string=None, doprint=None, output=False, enable=True, s
     return scu._printout(string=ansistring, doprint=doprint, output=output)
 
 
+# Alias certain colors functions -- not including white and black since poor practice on light/dark terminals
+def printred(s, **kwargs):
+    ''' Alias to print(colors.red(s)) '''
+    return print(ac.red(s, **kwargs))
+
+def printgreen(s, **kwargs):
+    ''' Alias to print(colors.green(s)) '''
+    return print(ac.green(s, **kwargs))
+
+def printblue(s, **kwargs):
+    ''' Alias to print(colors.blue(s)) '''
+    return print(ac.blue(s, **kwargs))
+
+def printcyan(s, **kwargs):
+    ''' Alias to print(colors.cyan(s)) '''
+    return print(ac.cyan(s, **kwargs))
+
+def printyellow(s, **kwargs):
+    ''' Alias to print(colors.yellow(s)) '''
+    return print(ac.yellow(s, **kwargs))
+
+def printmagenta(s, **kwargs):
+    ''' Alias to print(colors.magenta(s)) '''
+    return print(ac.magenta(s, **kwargs))
+
+
 def heading(string=None, *args, color=None, divider=None, spaces=None, spacesafter=None, minlength=None, maxlength=None, sep=' ', doprint=None, output=False, **kwargs):
     '''
     Create a colorful heading. If just supplied with a string (or list of inputs like print()),
@@ -799,6 +721,152 @@ def heading(string=None, *args, color=None, divider=None, spaces=None, spacesaft
     return colorize(color=color, string=fullstring, doprint=doprint, output=output, **kwargs)
 
 
+
+#%% Other
+
+__all__ += ['printv', 'slacknotification', 'printtologfile', 'percentcomplete', 'progressbar', 'capture']
+
+
+def printv(string, thisverbose=1, verbose=2, newline=True, indent=True):
+    '''
+    Optionally print a message and automatically indent. The idea is that
+    a global or shared "verbose" variable is defined, which is passed to
+    subfunctions, determining how much detail to print out.
+
+    The general idea is that verbose is an integer from 0-4 as follows:
+
+    * 0 = no printout whatsoever
+    * 1 = only essential warnings, e.g. suppressed exceptions
+    * 2 = standard printout
+    * 3 = extra debugging detail (e.g., printout on each iteration)
+    * 4 = everything possible (e.g., printout on each timestep)
+
+    Thus a very important statement might be e.g.
+
+    >>> sc.printv('WARNING, everything is wrong', 1, verbose)
+
+    whereas a much less important message might be
+
+    >>> sc.printv(f'This is timestep {i}', 4, verbose)
+
+    Version: 2016jan30
+    '''
+    if thisverbose>4 or verbose>4: print(f'Warning, verbosity should be from 0-4 (this message: {thisverbose}; current: {verbose})')
+    if verbose>=thisverbose: # Only print if sufficiently verbose
+        indents = '  '*thisverbose*bool(indent) # Create automatic indenting
+        if newline: print(indents+scu.flexstr(string)) # Actually print
+        else: print(indents+scu.flexstr(string)), # Actually print
+    return
+
+
+
+
+
+
+
+
+
+def slacknotification(message=None, webhook=None, to=None, fromuser=None, verbose=2, die=False):  # pragma: no cover
+    '''
+    Send a Slack notification when something is finished.
+
+    The webhook is either a string containing the webhook itself, or a plain text file containing
+    a single line which is the Slack webhook. By default it will look for the file
+    ".slackurl" in the user's home folder. The webhook needs to look something like
+    "https://hooks.slack.com/services/af7d8w7f/sfd7df9sb/lkcpfj6kf93ds3gj". Webhooks are
+    effectively passwords and must be kept secure! Alternatively, you can specify the webhook
+    in the environment variable SLACKURL.
+
+    Args:
+        message (str): The message to be posted.
+        webhook (str): See above
+        to (str): The Slack channel or user to post to. Channels begin with #, while users begin with @ (note: ignored by new-style webhooks)
+        fromuser (str): The pseudo-user the message will appear from (note: ignored by new-style webhooks)
+        verbose (bool): How much detail to display.
+        die (bool): If false, prints warnings. If true, raises exceptions.
+
+    **Example**::
+
+        sc.slacknotification('Long process is finished')
+        sc.slacknotification(webhook='/.slackurl', channel='@username', message='Hi, how are you going?')
+
+    What's the point? Add this to the end of a very long-running script to notify
+    your loved ones that the script has finished.
+
+    Version: 2018sep25
+    '''
+    try:
+        from requests import post # Simple way of posting data to a URL
+        from json import dumps # For sanitizing the message
+    except Exception as E:
+        errormsg = f'Cannot use Slack notification since imports failed: {str(E)}'
+        if die: raise ImportError(errormsg)
+        else:   print(errormsg)
+
+    # Validate input arguments
+    printv('Sending Slack message', 1, verbose)
+    if not webhook:  webhook    = os.path.expanduser('~/.slackurl')
+    if not to:       to       = '#general'
+    if not fromuser: fromuser = 'sciris-bot'
+    if not message:  message  = 'This is an automated notification: your notifier is notifying you.'
+    printv(f'Channel: {to} | User: {fromuser} | Message: {message}', 3, verbose) # Print details of what's being sent
+
+    # Try opening webhook as a file
+    if webhook.find('hooks.slack.com')>=0: # It seems to be a URL, let's proceed
+        slackurl = webhook
+    elif os.path.exists(os.path.expanduser(webhook)): # If not, look for it as a file
+        with open(os.path.expanduser(webhook)) as f: slackurl = f.read()
+    elif os.getenv('SLACKURL'): # See if it's set in the user's environment variables
+        slackurl = os.getenv('SLACKURL')
+    else:
+        slackurl = webhook # It doesn't seemt to be a URL but let's try anyway
+        errormsg = f'"{webhook}" does not seem to be a valid webhook string or file'
+        if die: raise ValueError(errormsg)
+        else:   print(errormsg)
+
+    # Package and post payload
+    try:
+        payload = '{"text": %s, "channel": %s, "username": %s}' % (dumps(message), dumps(to), dumps(fromuser))
+        printv(f'Full payload: {payload}', 4, verbose)
+        response = post(url=slackurl, data=payload)
+        printv(response, 3, verbose) # Optionally print response
+        printv('Message sent.', 2, verbose) # We're done
+    except Exception as E:
+        errormsg = f'Sending of Slack message failed: {repr(E)}'
+        if die: raise RuntimeError(errormsg)
+        else:   print(errormsg)
+    return
+
+
+def printtologfile(message=None, filename=None):
+    '''
+    Append a message string to a file specified by a filename name/path.
+
+    Note: in almost all cases, you are better off using Python's built-in logging
+    system rather than this function.
+    '''
+
+    # Set defaults
+    if message is None:
+        return # Return immediately if nothing to append
+    if filename is None:
+        import tempfile
+        tempdir = tempfile.gettempdir()
+        filename = os.path.join(tempdir, 'logfile') # Some generic filename that should work on *nix systems
+
+    # Try writing to file
+    try:
+        with open(filename, 'a') as f:
+            f.write('\n'+message+'\n') # Add a newline to the message.
+    except Exception as E: # pragma: no cover # Fail gracefully
+        print(f'Warning, could not write to logfile {filename}: {str(E)}')
+
+    return
+
+
+
+
+
 def percentcomplete(step=None, maxsteps=None, stepsize=1, prefix=None):
     '''
     Display progress.
@@ -815,39 +883,121 @@ def percentcomplete(step=None, maxsteps=None, stepsize=1, prefix=None):
         prefix = ' '
     elif scu.isnumber(prefix):
         prefix = ' '*prefix
-    onepercent = max(stepsize,round(maxsteps/100*stepsize)); # Calculate how big a single step is -- not smaller than 1
+    onepercent = max(stepsize,round(maxsteps/100*stepsize)) # Calculate how big a single step is -- not smaller than 1
     if not step%onepercent: # Does this value lie on a percent
         thispercent = round(step/maxsteps*100) # Calculate what percent it is
         print(prefix + '%i%%'% thispercent) # Display the output
-    return None
+    return
 
 
-def progressbar(i, maxiters, label='', length=30, empty='—', full='•', newline=False):
+def progressbar(i, maxiters, label='', every=1, length=30, empty='—', full='•', newline=False):
     '''
     Call in a loop to create terminal progress bar.
 
     Args:
-        i (int): current iteration
-        maxiters (int): maximum number of iterations
-        label (str): initial label to print
-        length (int): length of progress bar
-        empty (str): character for empty steps
-        full (str): character for empty steps
+        i        (int): current iteration
+        maxiters (int): maximum number of iterations (can also use an object with length)
+        label    (str): initial label to print
+        every    (int/float): if int, print every "every"th iteration (if 1, print all); if float and <1, print every maxiters*every iteration
+        length   (int): length of progress bar
+        empty    (str): character for not-yet-completed steps
+        full     (str): character for completed steps
+        newline  (str): character to print at the end of the line (default none)
 
-    **Example**::
+    **Examples**::
 
-        import pylab as pl
-        for i in range(100):
-            progressbar(i+1, 100)
+        for i in range(20):
+            sc.progressbar(i+1, 20)
             pl.pause(0.05)
 
+        x = np.arange(100)
+        for i in x:
+            sc.progressbar(i+1, x, every=0.1)
+            pl.pause(0.01)
+
     Adapted from example by Greenstick (https://stackoverflow.com/questions/3173320/text-progress-bar-in-the-console)
+
+    New in version 1.3.3: "every" argument
     '''
+    # Handle inputs
+    if hasattr(maxiters, '__len__'):
+        maxiters = len(maxiters)
     ending = None if newline else '\r'
-    pct = i/maxiters*100
+    if every < 1:
+        every = max(1, int(every*maxiters)) # Don't let it go below 1
+
+    # Calculate percent and handle zero case
+    if maxiters > 0:
+        pct = i/maxiters*100
+    else:
+        i = 1
+        maxiters = 1
+        pct = 100
     percent = f'{pct:0.0f}%'
+
+    # Assemble string
     filled = int(length*i//maxiters)
     bar = full*filled + empty*(length-filled)
-    print(f'\r{label} {bar} {percent}', end=ending)
-    if i == maxiters: print()
+
+    # Print
+    lastiter = (i == maxiters)
+    if not(i%every) or lastiter:
+        print(f'\r{label} {bar} {percent}', end=ending)
+        if lastiter:
+            print() # Newline at the end
+
     return
+
+
+class capture(UserString, str, redirect_stdout):
+    '''
+    Captures stdout (e.g., from ``print()``) as a variable.
+
+    Based on ``contextlib.redirect_stdout``, but saves the user the trouble of
+    defining and reading from an IO stream. Useful for testing the output of functions
+    that are supposed to print certain output.
+
+    **Examples**::
+
+        # Using with...as
+        with sc.capture() as txt1:
+            print('Assign these lines')
+            print('to a variable')
+
+        # Using start()...stop()
+        txt2 = sc.capture().start()
+        print('This works')
+        print('the same way')
+        txt2.stop()
+
+        print('txt1:')
+        print(txt1)
+        print('txt2:')
+        print(txt2)
+
+    New in version 1.3.3.
+    '''
+
+    def __init__(self, seq='', *args, **kwargs):
+        self._io = io.StringIO()
+        self.stdout = sys.stdout
+        UserString.__init__(self, seq=seq, *args, **kwargs)
+        redirect_stdout.__init__(self, self._io)
+        return
+
+    def __enter__(self, *args, **kwargs):
+        redirect_stdout.__enter__(self, *args, **kwargs)
+        return self
+
+    def __exit__(self, *args, **kwargs):
+        self.data += self._io.getvalue()
+        redirect_stdout.__exit__(self, *args, **kwargs)
+        return
+
+    def start(self):
+        self.__enter__()
+        return self
+
+    def stop(self):
+        self.__exit__(None, None, None)
+        return
\ No newline at end of file
