@@ -67,6 +67,10 @@ def test_find():
     found.vals = sc.findinds([2,3,6,3], 6)
     assert found.vals[0] == 2
 
+    print('Testing sc.count()')
+    found.count = sc.count([1,2,2,3], 2.0)
+    assert found.count == 2
+
     print('Testing sc.findfirst(), sc.findlast()')
     found.first = sc.findfirst(pl.rand(10))
     found.last = sc.findlast(pl.rand(10))
@@ -86,10 +90,31 @@ def test_find():
     assert found.data[-1] == 13
 
     print('Testing sc.sanitize()')
-    sanitized,inds = sc.sanitize(np.array([3, 4, np.nan, 8, 2, np.nan, np.nan, np.nan, 8]), returninds=True)
-    sanitized      = sc.sanitize(np.array([3, 4, np.nan, 8, 2, np.nan, np.nan, np.nan, 8]), replacenans=True)
-    sanitized      = sc.sanitize(np.array([3, 4, np.nan, 8, 2, np.nan, np.nan, np.nan, 8]), replacenans=0)
+    data = [3, 4, np.nan, 8, 2, np.nan, np.nan, 8]
+    sanitized, inds = sc.sanitize(data, returninds=True) # Remove NaNs
+    sc.sanitize(data, replacenans=True) # Replace NaNs using nearest neighbor interpolation
+    sc.sanitize(data, replacenans='nearest') # Eequivalent to replacenans=True
+    sc.sanitize(data, replacenans='linear') # Replace NaNs using linear interpolation
+    sc.sanitize(data, replacenans=0) # Replace NaNs with 0
+    data2d = np.random.rand(3,3)
+    data2d[1,1] = np.nan
+    data2d[2,2] = np.nan
+    with pytest.raises(ValueError):
+        sc.rmnans(data2d)
+    with pytest.raises(NotImplementedError):
+        sc.fillnans(data2d, replacenans='nearest')
+    filled = sc.fillnans(data2d, 100)
+    assert 200 < filled.sum() < 207
     found.sanitized = sanitized
+    found.filled = filled
+
+    print('Testing sc.numdigits()')
+    found.numdigits = sc.numdigits(1234)
+    found.numdigits_max = max(sc.numdigits([10, 200, 30000]))
+    found.numdigits_dec = sc.numdigits(0.01)
+    assert found.numdigits == 4
+    assert found.numdigits_max == 5
+    assert found.numdigits_dec == -2
 
     return found
 
@@ -153,18 +178,18 @@ def test_gauss2d(doplot=doplot):
     sc.heading('Testing Gaussian 2D smoothing')
 
     # Parameters
-    x = pl.rand(40)
-    y = pl.rand(40)
+    x = np.random.rand(40)
+    y = np.random.rand(40)
     z = 1-(x-0.5)**2 + (y-0.5)**2
 
     # Method 1 -- form grid
-    xi = pl.linspace(0,1,20)
-    yi = pl.linspace(0,1,20)
-    zi = sc.gauss2d(x, y, z, xi, yi, scale=0.1)
+    xi = np.linspace(0,1,20)
+    yi = np.linspace(0,1,20)
+    zi = sc.gauss2d(x, y, z, xi, yi, scale=0.1, grid=True)
 
     # Method 2 -- use points directly
-    xi2 = pl.rand(400)
-    yi2 = pl.rand(400)
+    xi2 = np.random.rand(400)
+    yi2 = np.random.rand(400)
     zi2 = sc.gauss2d(x, y, z, xi2, yi2, scale=0.1, grid=False)
 
     if doplot:
@@ -189,4 +214,4 @@ if __name__ == '__main__':
     gauss2d = test_gauss2d(doplot)
 
     sc.toc()
-    print('Done.')
\ No newline at end of file
+    print('Done.')
